<?php
/**
 * Plugin Name: FlowMattic
 * Plugin URI: https://www.persianscript.ir
 * Description: Workflow automation plugin for WordPress.
 * Version: 3.1.0
 * Requires at least: 5.0
 * Requires PHP:      7.2
 * Author: پرشین اسکریپت
 * Author URI: https://www.persianscript.ir
 * Textdomain: flowmattic
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

// Plugin version.
if ( ! defined( 'FLOWMATTIC_VERSION' ) ) {
	define( 'FLOWMATTIC_VERSION', '3.1.0' );
}

// Plugin Root File.
if ( ! defined( 'FLOWMATTIC_PLUGIN_FILE' ) ) {
	define( 'FLOWMATTIC_PLUGIN_FILE', __FILE__ );
}

// Plugin Folder Path.
if ( ! defined( 'FLOWMATTIC_PLUGIN_DIR' ) ) {
	define( 'FLOWMATTIC_PLUGIN_DIR', wp_normalize_path( plugin_dir_path( FLOWMATTIC_PLUGIN_FILE ) ) );
}

// Plugin Folder URL.
if ( ! defined( 'FLOWMATTIC_PLUGIN_URL' ) ) {
	define( 'FLOWMATTIC_PLUGIN_URL', plugin_dir_url( FLOWMATTIC_PLUGIN_FILE ) );
}

// Plugin Root File.
if ( ! defined( 'FLOWMATTIC_APP_URL' ) ) {
	define( 'FLOWMATTIC_APP_URL', WP_CONTENT_URL . '/flowmattic-apps' );
}

if ( ! class_exists( 'FlowMattic' ) ) {

	/**
	 * Main FlowMattic Class.
	 *
	 * @since 1.0
	 */
	class FlowMattic {

		/**
		 * The one, true instance of this object.
		 *
		 * @since 1.0
		 * @static
		 * @access private
		 * @var object
		 */
		private static $instance;

		/**
		 * Workflow class instance.
		 *
		 * @since 1.0
		 * @access public
		 * @var object
		 */
		public $workflow;

		/**
		 * Integrations.
		 *
		 * @access public
		 * @since 1.0
		 * @var array
		 */
		public $integrations = array();

		/**
		 * Connects.
		 *
		 * @access public
		 * @since 3.0
		 * @var array
		 */
		public $connects = array();

		/**
		 * Custom Apps.
		 *
		 * @access public
		 * @since 3.0
		 * @var array
		 */
		public $custom_apps = array();

		/**
		 * Workflows.
		 *
		 * @access public
		 * @since 1.0
		 * @var array
		 */
		public $workflows_db = array();

		/**
		 * Tasks.
		 *
		 * @access public
		 * @since 1.0
		 * @var array
		 */
		public $tasks_db = array();

		/**
		 * Connects.
		 *
		 * @access public
		 * @since 3.0
		 * @var array
		 */
		public $connects_db = array();

		/**
		 * Custom Apps.
		 *
		 * @access public
		 * @since 3.0
		 * @var array
		 */
		public $custom_apps_db = array();
		/**
		 * Updater.
		 *
		 * @access public
		 * @since 1.0
		 * @var array
		 */
		public $updater = array();

		/**
		 * Applications.
		 *
		 * @access public
		 * @since 1.0
		 * @var array
		 */
		public $apps = array();

		/**
		 * Creates or returns an instance of this class.
		 *
		 * @since 1.0
		 * @static
		 * @access public
		 */
		public static function get_instance() {

			// If an instance hasn't been created and set to $instance create an instance and set it to $instance.
			if ( null === self::$instance ) {
				self::$instance = new FlowMattic();
			}
			return self::$instance;
		}

		/**
		 * Initializes the plugin by setting localization, hooks, filters,
		 * and administrative functions.
		 *
		 * @since 1.0
		 * @access private
		 */
		private function __construct() {
			// Instantiate the updater.
			$this->updater = new FlowMattic_Updater( FLOWMATTIC_PLUGIN_FILE );

			// Instantiate the workflow class.
			$this->workflow = new FlowMattic_Workflow();

			// Instantiate the connects class.
			$this->connects = new FlowMattic_Connects();

			// Instantiate the custom_apps class.
			$this->custom_apps = new FlowMattic_Custom_Apps();

			// Instantiate the workflows database.
			$this->workflows_db = new FlowMattic_Database_Workflows();

			// Instantiate the connects database.
			$this->connects_db = new FlowMattic_Database_Connects();

			// Instantiate the custom apps database.
			$this->custom_apps_db = new FlowMattic_Database_Custom_Apps();

			// Instantiate the tasks database.
			$this->tasks_db = new FlowMattic_Database_Tasks();

			// Initialise admin.
			new FlowMattic_Admin();

			$this->apps = new FlowMattic_Applications();
		}

		/**
		 * Create database tables.
		 *
		 * @since 1.0
		 * @access public
		 */
		public function create_tables() {
			$flowmattic_database = new FlowMattic_Database();
			$flowmattic_database->create_tables();
		}

		/**
		 * Check license validation.
		 *
		 * @since 1.0
		 * @access public
		 * @return bool|object
		 */
		public function check_license() {
			$license_key   = get_option( 'flowmattic_license_key', '' );
			$license_email = get_option( 'flowmattic_license_email', '' );

			FlowMattic_Updater::addOnDelete(
				function() {
					delete_option( 'flowmattic_license_key' );
				}
			);

			if ( FlowMattic_Updater::CheckWPPlugin( $license_key, $license_email, $license_message, $response_object, FLOWMATTIC_PLUGIN_FILE ) ) {
				return $response_object;
			} else {
				return false;
			}
		}
	}
}

/**
 * Include the Flowmattic autoloader class.
 */
require_once wp_normalize_path( FLOWMATTIC_PLUGIN_DIR . '/inc/class-flowmattic-autoload.php' );

/**
 * Instantiate the autoloader.
 */
new FlowMattic_Autoload();

/**
 * Instantiates the FlowMattic class.
 * Make sure the class is properly set-up.
 * The FlowMattic class is a singleton
 * so we can directly access the one true FlowMattic object using this function.
 *
 * @return object FlowMattic
 */
function wp_flowmattic() {
	return FlowMattic::get_instance();
}

/**
 * Instantiate FlowMattic class.
 *
 * @since 1.0
 * @return void
 */
function infi_activate__flowmattic() {
	// Include the helpers.
	require_once wp_normalize_path( FLOWMATTIC_PLUGIN_DIR . '/inc/helpers.php' );

	// Initiate the FlowMattic.
	wp_flowmattic();

	// Check if version is different, reset the option to create database.
	if ( FLOWMATTIC_VERSION !== get_option( 'flowmattic_version' ) ) {
		delete_option( 'flowmattic_data_tables_created' );
		update_option( 'flowmattic_version', FLOWMATTIC_VERSION );
	}

	// Create database tables if not exist.
	$flowmattic_data_tables_created = get_option( 'flowmattic_data_tables_created', false );
	if ( ! $flowmattic_data_tables_created ) {
		wp_flowmattic()->create_tables();
	}
}
add_action( 'after_setup_theme', 'infi_activate__flowmattic', 11 );
