<?php
/**
 * Admin page template for connects.
 *
 * @package FlowMattic
 * @since 3.0
 */

FlowMattic_Admin::loader();
?>
<div class="wrap flowmattic-wrap about-wrap">
	<div class="flowmattic-wrapper d-flex">
		<?php FlowMattic_Admin::header(); ?>
		<div class="flowmattic-dashboard-content container m-0 ps-3">
			<div class="row">
				<?php
				$this_user = wp_get_current_user();
				$email     = $this_user->user_email;

				$license_key = get_option( 'flowmattic_license_key', '' );

				// Get all sources.
				$connect_sources = (array) flowmattic_get_integrations( 'connects' );

				// Get external connects.
				$external_connects = flowmattic_get_connects();

				$all_connects = array();
				if ( current_user_can( 'manage_options' ) ) {
					$all_connects = wp_flowmattic()->connects_db->get_all();
				}

				$connects_html = '';

				if ( empty( $all_connects ) ) {
					ob_start();
					?>
						<div class="card border-light mw-100 p-3">
							<div class="card-body">
								<h5 class="card-title text-dark"><?php esc_html_e( 'Connect and manage your API authentications', 'flowmattic' ); ?></h5>
								<p class="card-text text-secondary">
									<?php esc_html_e( 'FlowMattic connects is built to keep your API authentications centralized. You can use these connects in API module or the app integration or custom app.', 'flowmattic' ); ?>
									<br>
									<?php esc_html_e( 'No coding skills required. You just need to get the API information required for your app.', 'flowmattic' ); ?>
								</p>
								<?php
									$button_type  = '';
									$button_attr  = 'data-toggle="modal" data-target="#choose-connect-settings-modal"';
									$button_attr2 = 'data-toggle="modal" data-target="#choose-connect-external-modal"';
									$button_class = '';
									$button_url   = 'javascript:void(0)';

								if ( ( empty( $connect_sources ) || ( isset( $connect_sources['status'] ) || isset( $connect_sources['message'] ) ) ) || '' === $license_key ) {
									$button_type  = 'disabled';
									$button_class = 'needs-registration';
									$button_attr  = '';
								} else {
									if ( ! empty( $external_connects ) ) {
										?>
										<a href="<?php echo $button_url; ?>" <?php echo esc_attr( $button_type ); ?>  class="btn btn-md btn-success d-inline-flex align-items-center justify-content-center me-2 <?php echo $button_class; ?>" <?php echo $button_attr2; ?>>
											<span class="dashicons dashicons-plus-alt2 d-inline-block pe-3 ps-0 me-1"></span>
											<?php echo esc_attr__( 'Connect Integration', 'flowmattic' ); ?>
										</a>
										<?php
									}
									?>
									<a href="<?php echo $button_url; ?>" <?php echo esc_attr( $button_type ); ?>  class="btn btn-md btn-primary d-inline-flex align-items-center justify-content-center <?php echo $button_class; ?>" <?php echo $button_attr; ?>>
										<span class="dashicons dashicons-plus-alt2 d-inline-block pe-3 ps-0 me-1"></span>
									<?php echo esc_attr__( 'Connect New API', 'flowmattic' ); ?>
									</a>
									<?php
								}
								?>
							</div>
						</div>
					<?php
					$connects_html = ob_get_clean();
					} else {
						$nonce = wp_create_nonce( 'flowmattic-connect-edit' );

						$connect_statuses = array(
							'live'  => 0,
							'draft' => 0,
						);

						$auth_types = array(
							'basic'  => esc_attr__( 'Basic', 'flowmattic' ),
							'api'    => esc_attr__( 'API Key', 'flowmattic' ),
							'bearer' => esc_attr__( 'Bearer Token', 'flowmattic' ),
							'oauth'  => esc_attr__( 'OAuth 2.0', 'flowmattic' ),
						);
						ob_start();
						?>
					<table class="table bg-white table-hover">
						<thead>
							<tr>
								<th scope="col"><span class="ps-2">#</span></th>
								<th scope="col"><?php esc_html_e( 'Connect Name', 'flowmattic' ); ?></th>
								<th scope="col"><?php esc_html_e( 'Auth Type', 'flowmattic' ); ?></th>
								<th scope="col"><?php esc_html_e( 'Source', 'flowmattic' ); ?></th>
								<th scope="col"><?php esc_html_e( 'Created At', 'flowmattic' ); ?></th>
								<th scope="col" class="text-center"><?php esc_html_e( 'Actions', 'flowmattic' ); ?></th>
							</tr>
						</thead>
						<tbody>
						<?php
						foreach ( $all_connects as $key => $connect_item ) {
							$connect_id       = $connect_item->id;
							$connect_settings = json_decode( base64_decode( $connect_item->connect_settings ) );

							// Check if external connect.
							$external_connect = ( isset( $connect_settings->is_external ) ) ? flowmattic_get_connects( $connect_settings->external_slug ) : false;

							$auth_type = ! empty( $external_connect ) ? $external_connect['fm_auth_type'] : $connect_settings->fm_auth_type;
							$badge     = ! empty( $external_connect ) ? '<span class="badge bg-primary text-black">Integration</span>' : '<span class="badge bg-secondary text-black">Custom</span>';
							?>
							<tr class="connect-item-<?php echo esc_attr( $connect_id ); ?>">
								<th scope="row"><span class="ps-2"><?php echo esc_attr( $connect_id ); ?></span></th>
								<td class="connect-name"><?php echo esc_attr( $connect_item->connect_name ); ?></td>
								<td><small><?php echo esc_attr( $auth_types[ $auth_type ] ); ?></small></td>
								<td><small><?php echo $badge; ?></small></td>
								<td><small><?php echo esc_attr( date_i18n( 'd-m-Y h:i A', strtotime( $connect_item->connect_time ) ) ); ?></small></td>
								<td>
									<div class="d-flex justify-content-center">
										<a href="javascript:void(0);" data-connect-id="<?php echo esc_attr( $connect_id ); ?>" data-connect-name="<?php echo esc_attr( $connect_item->connect_name ); ?>" class="btn-rename-connect btn btn-primary-outline btn-sm d-inline-flex align-items-center justify-content-center me-3 p-0" style="width: 28px; height: 28px;" data-toggle="tooltip" title="<?php echo esc_html__( 'Rename Connect', 'flowmattic' ); ?>">
											<span class="screen-reader-text"><?php echo esc_html__( 'Rename Connect', 'flowmattic' ); ?></span>
											<svg width="18" height="18" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" data-reactroot=""><path stroke-width="1" stroke="#221b38" fill="none" d="M17.82 2.29L5.01 15.11L2 22L8.89 18.99L21.71 6.18C22.1 5.79 22.1 5.16 21.71 4.77L19.24 2.3C18.84 1.9 18.21 1.9 17.82 2.29Z" clip-rule="evenodd" fill-rule="evenodd"></path><path stroke-width="1" stroke="#221b38" fill="none" d="M5.01 15.11L8.89 18.99L2 22L5.01 15.11Z"></path><path stroke-width="1" stroke="#221b38" fill="none" d="M19.23 8.65999L15.34 4.76999L17.81 2.29999C18.2 1.90999 18.83 1.90999 19.22 2.29999L21.69 4.76999C22.08 5.15999 22.08 5.78999 21.69 6.17999L19.23 8.65999Z"></path></svg>
										</a>
										<a href="javascript:void(0);" data-connect-id="<?php echo esc_attr( $connect_id ); ?>"  class="btn-edit-connect btn btn-primary-outline btn-sm d-inline-flex align-items-center justify-content-center me-3 p-0" style="width: 28px; height: 28px;" data-toggle="tooltip" title="<?php echo esc_html__( 'Change Settings', 'flowmattic' ); ?>">
											<span class="screen-reader-text"><?php echo esc_html__( 'Change Settings', 'flowmattic' ); ?></span>
											<svg xmlns="http://www.w3.org/2000/svg" width="18" height="18" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="1.2" stroke-linecap="round" stroke-linejoin="round" class="feather feather-settings"><circle cx="12" cy="12" r="3"></circle><path d="M19.4 15a1.65 1.65 0 0 0 .33 1.82l.06.06a2 2 0 0 1 0 2.83 2 2 0 0 1-2.83 0l-.06-.06a1.65 1.65 0 0 0-1.82-.33 1.65 1.65 0 0 0-1 1.51V21a2 2 0 0 1-2 2 2 2 0 0 1-2-2v-.09A1.65 1.65 0 0 0 9 19.4a1.65 1.65 0 0 0-1.82.33l-.06.06a2 2 0 0 1-2.83 0 2 2 0 0 1 0-2.83l.06-.06a1.65 1.65 0 0 0 .33-1.82 1.65 1.65 0 0 0-1.51-1H3a2 2 0 0 1-2-2 2 2 0 0 1 2-2h.09A1.65 1.65 0 0 0 4.6 9a1.65 1.65 0 0 0-.33-1.82l-.06-.06a2 2 0 0 1 0-2.83 2 2 0 0 1 2.83 0l.06.06a1.65 1.65 0 0 0 1.82.33H9a1.65 1.65 0 0 0 1-1.51V3a2 2 0 0 1 2-2 2 2 0 0 1 2 2v.09a1.65 1.65 0 0 0 1 1.51 1.65 1.65 0 0 0 1.82-.33l.06-.06a2 2 0 0 1 2.83 0 2 2 0 0 1 0 2.83l-.06.06a1.65 1.65 0 0 0-.33 1.82V9a1.65 1.65 0 0 0 1.51 1H21a2 2 0 0 1 2 2 2 2 0 0 1-2 2h-.09a1.65 1.65 0 0 0-1.51 1z"></path></svg>
										</a>
										<a href="javascript:void(0);" data-connect-id="<?php echo esc_attr( $connect_id ); ?>"  class="btn-delete-connect btn btn-primary-outline text-danger btn-sm d-inline-flex align-items-center justify-content-center p-0" style="width: 28px; height: 28px;" data-toggle="tooltip" title="<?php echo esc_html__( 'Delete Connect', 'flowmattic' ); ?>">
											<span class="screen-reader-text"><?php echo esc_html__( 'Delete', 'flowmattic' ); ?></span>
											<svg width="18" height="18" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg" data-reactroot=""><path stroke-width="1" stroke="currentColor" fill="none" d="M16.13 22H7.87C7.37 22 6.95 21.63 6.88 21.14L5 8H19L17.12 21.14C17.05 21.63 16.63 22 16.13 22Z"></path><path stroke-width="1" stroke="currentColor" d="M3.5 8H20.5"></path><path stroke-width="1" stroke="currentColor" d="M10 12V18"></path><path stroke-width="1" stroke="currentColor" d="M14 12V18"></path><path stroke-width="1" stroke="currentColor" fill="none" d="M16 5H8L9.7 2.45C9.89 2.17 10.2 2 10.54 2H13.47C13.8 2 14.12 2.17 14.3 2.45L16 5Z"></path><path stroke-width="1" stroke="currentColor" d="M3 5H21"></path></svg>
										</a>
									</div>
								</td>
							</tr>
							<?php
						}
						?>
						</tbody>
					</table>
						<?php
						$connects_html .= ob_get_clean();
					}
					?>
				<div class="flowmattic-container flowmattic-dashboard m-0">
					<div class="row flex-row-reverse flex-xl-row-reverse flex-sm-column">
						<div class="col-sm-12 fm-connects-list-container ps-4 pe-4">
							<div class="fm-connect-task-header d-flex mb-4 mt-4 justify-content-between">
								<h3 class="fm-connect-heading m-0 d-flex align-items-center">
									<?php echo esc_attr__( 'FlowMattic Connects', 'flowmattic' ); ?>
								</h3>
								<div class="flowmattic-connects-header-actions">
									<?php
									$button_type  = '';
									$button_attr  = 'data-toggle="modal" data-target="#choose-connect-settings-modal"';
									$button_attr2 = 'data-toggle="modal" data-target="#choose-connect-external-modal"';
									$button_class = '';
									$button_url   = 'javascript:void(0)';

									if ( ( empty( $connect_sources ) || ( isset( $connect_sources['status'] ) || isset( $connect_sources['message'] ) ) ) || '' === $license_key ) {
										$button_type  = 'disabled';
										$button_class = 'needs-registration';
										$button_attr  = '';
									} else {
										if ( ! empty( $external_connects ) ) {
											?>
											<a href="<?php echo $button_url; ?>" <?php echo esc_attr( $button_type ); ?>  class="btn btn-md btn-success d-inline-flex align-items-center justify-content-center <?php echo $button_class; ?>" <?php echo $button_attr2; ?>>
												<span class="dashicons dashicons-plus-alt2 d-inline-block pe-3 ps-0 me-1"></span>
												<?php echo esc_attr__( 'Connect Integration', 'flowmattic' ); ?>
											</a>
											<?php
										}
										?>
										<a href="<?php echo $button_url; ?>" <?php echo esc_attr( $button_type ); ?>  class="btn btn-md btn-primary d-inline-flex align-items-center justify-content-center <?php echo $button_class; ?>" <?php echo $button_attr; ?>>
											<span class="dashicons dashicons-plus-alt2 d-inline-block pe-3 ps-0 me-1"></span>
											<?php echo esc_attr__( 'Connect New API', 'flowmattic' ); ?>
										</a>
										<?php
									}
									?>
								</div>
							</div>
							<div class="connects-nav navbar mt-3 mb-3 bg-light">
								<span class="navbar-text ps-3">
									<?php esc_html_e( 'Connect and manage your custom API connections here.', 'flowmattic' ); ?>
									<a href="https://flowmattic.com/features/flowmattic-connects/" target="_blank" class="text-decoration-none"><?php esc_html_e( 'Learn more' ); ?> <svg width="16" height="16" viewBox="0 0 16 16" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M6.31909 5H11.3191V10H10.3191V6.70704L5.70711 11.3191L5 10.612L9.61192 6H6.31909V5Z" fill="currentColor"></path></svg></a>
								</span>
							</div>
							<div class="fm-connects-list">
								<?php
								if ( '' === $license_key ) {
									?>
									<div class="card border-light mw-100">
										<div class="card-body text-center">
											<div class="alert alert-primary" role="alert">
												<?php echo esc_html__( 'License key not registered. Please register your license first to use connects.', 'flowmattic' ); ?>
											</div>
										</div>
									</div>
									<?php
								} elseif ( ( empty( $connect_sources ) || ( isset( $connect_sources['status'] ) || isset( $connect_sources['message'] ) ) ) || '' === $license_key ) {
									?>
										<div class="card border-light mw-100">
											<div class="card-body text-center">
												<div class="alert alert-primary p-4 m-5 text-center" role="alert">
													<?php echo $connect_sources['message']; ?>
												</div>
											</div>
										</div>
									<?php
								} else {
									echo $connects_html;
								}
								?>
							</div>
						</div>
					</div>
				</div>
				<!-- Integration Connect Modal -->
				<div class="modal fade" id="choose-connect-external-modal" data-backdrop="static" data-keyboard="false" tabindex="-1" aria-labelledby="choose-connect-external-modal-label" aria-hidden="true">
					<div class="modal-dialog modal-xl modal-dialog-centered">
						<div class="modal-content">
							<div class="modal-header">
								<h5 class="modal-title" id="choose-connect-external-modal-label"><?php esc_html_e( 'Connect Integration', 'flowmattic' ); ?></h5>
								<button type="button" class="btn-close" data-dismiss="modal" aria-label="<?php esc_html_e( 'Close', 'flowmattic' ); ?>"></button>
							</div>
							<div class="modal-body">
								<div class="row row-cols-1 row-cols-md-6 g-4">
									<?php
									if ( ! empty( $external_connects ) ) {
										foreach ( $external_connects as $slug => $connect_app ) {
											$name      = $connect_app['name'];
											$icon      = $connect_app['icon'];
											$auth_type = $connect_app['fm_auth_type'];
											?>
											<div class="col">
												<a href="javascript:void(0);" class="text-decoration-none text-reset external-connect-auth-button connect-<?php echo esc_attr( $auth_type ); ?>-button" data-connect-slug="<?php echo esc_attr( $slug ); ?>">
													<div class="card m-0 p-0">
														<div class="card-body px-5">
															<img src="<?php echo esc_attr( $icon ); ?>">
														</div>
														<div class="card-footer text-center form-text bg-light px-1">
															<?php echo esc_attr( $name ); ?>
														</div>
													</div>
												</a>
											</div>
											<?php
										}
									}
									?>
								</div>
							</div>
						</div>
					</div>
				</div>
				<!-- New Connect Modal -->
				<div class="modal fade" id="choose-connect-settings-modal" data-backdrop="static" data-keyboard="false" tabindex="-1" aria-labelledby="choose-connect-settings-modal-label" aria-hidden="true">
					<div class="modal-dialog modal-lg modal-dialog-centered">
						<form id="fm-connect-form" novalidate>
							<div class="modal-content">
								<div class="modal-header">
									<h5 class="modal-title" id="choose-connect-settings-modal-label"><?php esc_html_e( 'New Connect', 'flowmattic' ); ?></h5>
									<button type="button" class="btn-close" data-dismiss="modal" aria-label="<?php esc_html_e( 'Close', 'flowmattic' ); ?>"></button>
								</div>
								<div class="modal-body">
									<div class="mb-3">
										<label for="connect_name" class="form-label"><?php esc_html_e( 'Connect Name', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></label>
										<input type="search" name="connect_name" class="form-control fm-textarea" id="connect_name" required>
										<div class="form-text"><?php esc_html_e( 'Name the connection, prefer API provider name for better understanding.', 'flowmattic' ); ?></div>
									</div>
									<div class="mb-3 border-bottom">
										<label for="fm-auth-type" class="form-label"><?php esc_html_e( 'Authentication Type', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></label>
										<select id="fm-auth-type" name="fm_auth_type" class="form-control fm-select border w-100" title="<?php esc_html_e( 'Authentication type', 'flowmattic' ); ?>" data-live-search="true" required>
											<option value="basic" data-subtext="<?php esc_html_e( 'Username-password based access using HTTP headers.', 'flowmattic' ); ?>">Basic</option>
											<option value="api" data-subtext="<?php esc_html_e( 'Single secret token for API request authorization.', 'flowmattic' ); ?>">API Key</option>
											<option value="bearer" data-subtext="<?php esc_html_e( 'Authenticate API using an access key, such as a JSON Web Token (JWT).', 'flowmattic' ); ?>">Bearer Token</option>
											<option value="oauth" data-subtext="<?php esc_html_e( 'Protocol for granting scoped API access via tokens.', 'flowmattic' ); ?>">OAuth 2.0</option>
										</select>
										<div class="mt-2 mb-3 form-text">
											<?php esc_html_e( 'The authorization header will be automatically generated when you send the request. Learn more about', 'flowmattic' ); ?>
											<a href="https://learning.postman.com/docs/sending-requests/authorization/" target="_blank" class="text-decoration-none">
												<?php esc_html_e( 'authorization', 'flowmattic' ); ?>
												<svg width="16" height="16" viewBox="0 0 16 16" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M6.31909 5H11.3191V10H10.3191V6.70704L5.70711 11.3191L5 10.612L9.61192 6H6.31909V5Z" fill="#6B6B6B"></path></svg>
											</a>
										</div>
									</div>
									<div class="auth-details-form">
										<div class="alert alert-primary p-4 m-5 text-center" role="alert">
											<?php esc_html_e( 'Select the Authentication Type to get started', 'flowmattic' ); ?>
										</div>
									</div>
								</div>
							</div>
						</form>
					</div>
				</div>
				<!-- Rename Connect Modal -->
				<div class="modal fade" id="fm-connect-rename-modal" data-backdrop="static" data-keyboard="false" tabindex="-1" aria-labelledby="fm-connect-rename-modal-label" aria-hidden="true">
					<div class="modal-dialog modal-md modal-dialog-centered">
						<form id="fm-connect-rename" class="w-100" novalidate>
							<input class="hidden" name="connect_id" type="hidden" value=""/>
							<div class="modal-content">
								<div class="modal-header">
									<h5 class="modal-title" id="fm-connect-rename-modal-label"><?php esc_html_e( 'Rename Connect', 'flowmattic' ); ?></h5>
									<button type="button" class="btn-close" data-dismiss="modal" aria-label="<?php esc_html_e( 'Close', 'flowmattic' ); ?>"></button>
								</div>
								<div class="modal-body">
									<div class="mb-3">
										<label for="connect_name" class="form-label"><?php esc_html_e( 'Connect Name', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></label>
										<input type="search" name="connect_name" class="form-control fm-textarea" id="connect_name" required>
										<div class="form-text"><?php esc_html_e( 'Name the connection, prefer API provider name for better understanding.', 'flowmattic' ); ?></div>
									</div>
									<p><button type="submit" class="btn btn-rename-auth btn-primary"><?php esc_html_e( 'Rename', 'flowmattic' ); ?></button></p>
								</div>
							</div>
						</form>
					</div>
				</div>
				<!-- Edit Connect Modal -->
				<div class="modal fade" id="edit-connect-settings-modal" data-backdrop="static" data-keyboard="false" tabindex="-1" aria-labelledby="edit-connect-settings-modal-label" aria-hidden="true">
					<div class="modal-dialog modal-lg modal-dialog-centered">
						<form id="fm-connect-edit-form" novalidate>
							<input class="hidden" name="connect_id" type="hidden" value=""/>
							<div class="modal-content">
								<div class="modal-header">
									<h5 class="modal-title" id="edit-connect-settings-modal-label"><?php esc_html_e( 'Edit Connect:', 'flowmattic' ); ?> <span class="connect-name"></span></h5>
									<button type="button" class="btn-close" data-dismiss="modal" aria-label="<?php esc_html_e( 'Close', 'flowmattic' ); ?>"></button>
								</div>
								<div class="modal-body">
									<div class="mb-3">
										<label for="connect_name" class="form-label"><?php esc_html_e( 'Connect Name', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></label>
										<input type="search" name="connect_name" class="form-control fm-textarea" id="connect_name" required>
										<div class="form-text"><?php esc_html_e( 'Name the connection, prefer API provider name for better understanding.', 'flowmattic' ); ?></div>
									</div>
									<div class="mb-3 border-bottom">
										<label for="fm-auth-type" class="form-label"><?php esc_html_e( 'Authentication Type', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></label>
										<select disabled id="fm-auth-type" name="fm_auth_type" class="form-control fm-select border w-100" title="<?php esc_html_e( 'Authentication type', 'flowmattic' ); ?>" data-live-search="true" required>
											<option value="basic" data-subtext="<?php esc_html_e( 'Username-password based access using HTTP headers.', 'flowmattic' ); ?>">Basic</option>
											<option value="api" data-subtext="<?php esc_html_e( 'Single secret token for API request authorization.', 'flowmattic' ); ?>">API Key</option>
											<option value="bearer" data-subtext="<?php esc_html_e( 'Authenticate API using an access key, such as a JSON Web Token (JWT).', 'flowmattic' ); ?>">Bearer Token</option>
											<option value="oauth" data-subtext="<?php esc_html_e( 'Protocol for granting scoped API access via tokens.', 'flowmattic' ); ?>">OAuth 2.0</option>
										</select>
										<div class="mt-2 mb-3 form-text">
											<?php esc_html_e( 'The authorization header will be automatically generated when you send the request. Learn more about', 'flowmattic' ); ?>
											<a href="https://learning.postman.com/docs/sending-requests/authorization/" target="_blank" class="text-decoration-none">
												<?php esc_html_e( 'authorization', 'flowmattic' ); ?>
												<svg width="16" height="16" viewBox="0 0 16 16" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M6.31909 5H11.3191V10H10.3191V6.70704L5.70711 11.3191L5 10.612L9.61192 6H6.31909V5Z" fill="#6B6B6B"></path></svg>
											</a>
										</div>
									</div>
									<div class="auth-details-form">
										<div class="alert alert-primary p-4 m-5 text-center" role="alert">
											<?php esc_html_e( 'Loading settings...', 'flowmattic' ); ?>
										</div>
									</div>
								</div>
							</div>
						</form>
					</div>
				</div>
				<!-- Connect Basic -->
				<script type="text/html" id="connect-type-basic-form">
					<div class="form-group bg-light p-3 pb-1 mb-3 api-url w-100">
						<div class="mb-3 auth_api_key">
							<h4 class="fm-input-title"><?php esc_attr_e( 'API Key / Username', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></h4>
							<input class="form-control fm-textarea w-100" name="auth_api_key" type="search" autocomplete="new-password" required/>
						</div>
						<div class="mb-3 auth_api_secret">
							<h4 class="fm-input-title"><?php esc_attr_e( 'API Secret / Password', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></h4>
							<input class="form-control fm-textarea w-100" name="auth_api_secret" type="search" autocomplete="new-password" required/>
						</div>
					</div>
					<p><button type="submit" class="btn btn-save-auth btn-primary"><?php esc_html_e( 'Save Connect', 'flowmattic' ); ?></button></p>
				</script>
				<!-- Connect API Key -->
				<script type="text/html" id="connect-type-api-form">
					<div class="form-group bg-light p-3 pb-1 mb-3 api-url w-100">
						<div class="mb-3 auth_api_key">
							<h4 class="fm-input-title"><?php esc_attr_e( 'API Key Name', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></h4>
							<input class="form-control fm-textarea w-100" name="auth_api_key" type="search" autocomplete="new-password" placeholder="apikey" required/>
						</div>
						<div class="mb-3 auth_api_value">
							<h4 class="fm-input-title"><?php esc_attr_e( 'API Key Value', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></h4>
							<input class="form-control fm-textarea w-100" name="auth_api_value" type="search" autocomplete="new-password" placeholder="Eg: 338c5d58ffe9ea713c3e52ad1443e749" required/>
						</div>
						<div class="mb-3 auth_api_addto">
							<h4 class="fm-input-title"><?php esc_attr_e( 'Add API Key to', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></h4>
							<select class="form-control fm-select border bg-white w-100 mw-100" name="auth_api_addto" required>
								<option value="query">Query Param</option>
								<option value="header">Header</option>
							</select>
						</div>
					</div>
					<p><button type="submit" class="btn btn-save-auth btn-primary"><?php esc_html_e( 'Save Connect', 'flowmattic' ); ?></button></p>
				</script>
				<!-- Connect API Key - External -->
				<script type="text/html" id="connect-type-api-form-external">
					<div class="form-group bg-light p-3 pb-1 mb-3 api-url w-100">
						<div class="mb-3 auth_api_value">
							<h4 class="fm-input-title"><?php esc_attr_e( 'API Key Value', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></h4>
							<input class="form-control fm-textarea w-100" name="auth_api_value" type="search" autocomplete="new-password" placeholder="Eg: 338c5d58ffe9ea713c3e52ad1443e749" required/>
						</div>
					</div>
					<p><button type="submit" class="btn btn-save-auth btn-primary"><?php esc_html_e( 'Save Connect', 'flowmattic' ); ?></button></p>
				</script>
				<!-- Connect Bearer Token -->
				<script type="text/html" id="connect-type-bearer-form">
					<div class="form-group bg-light p-3 pb-1 mb-3 api-url w-100">
						<div class="mb-3 auth_bearer_token">
							<h4 class="fm-input-title"><?php esc_attr_e( 'Bearer Token', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></h4>
							<input class="form-control fm-textarea w-100" name="auth_bearer_token" type="search" autocomplete="new-password" placeholder="Enter token here" required/>
						</div>
					</div>
					<p><button type="submit" class="btn btn-save-auth btn-primary"><?php esc_html_e( 'Save Connect', 'flowmattic' ); ?></button></p>
				</script>
				<!-- Connect OAuth -->
				<script type="text/html" id="connect-type-oauth-form">
					<input class="hidden" name="grant_type" type="hidden" value="authorization_code"/>
					<div class="form-group bg-light p-3 pb-1 mb-3 api-url w-100">
						<div class="mb-3">
							<h4 class="fm-input-title">
								<?php esc_attr_e( 'Callback URL', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small>
								<a href="#" data-toggle="tooltip" data-placement="top" title="This is the callback URL that you will be redirected to, after your application is authorized. FlowMattic uses this to extract the authorization code or access token. The callback URL should match the one you use during the application registration process.">
									<svg width="12" height="12" viewBox="0 0 12 12" fill="none" xmlns="http://www.w3.org/2000/svg"><path d="M6 4.5C6.41421 4.5 6.75 4.16421 6.75 3.75C6.75 3.33579 6.41421 3 6 3C5.58579 3 5.25 3.33579 5.25 3.75C5.25 4.16421 5.58579 4.5 6 4.5Z" fill="#6B6B6B"></path><path d="M4.5 9V8H5.5V6H4.5V5H6.5V8H7.5V9H4.5Z" fill="#6B6B6B"></path><path fill-rule="evenodd" clip-rule="evenodd" d="M12 6C12 9.31371 9.31371 12 6 12C2.68629 12 0 9.31371 0 6C0 2.68629 2.68629 0 6 0C9.31371 0 12 2.68629 12 6ZM11 6C11 8.76142 8.76142 11 6 11C3.23858 11 1 8.76142 1 6C1 3.23858 3.23858 1 6 1C8.76142 1 11 3.23858 11 6Z" fill="#6B6B6B"></path></svg>
								</a>
							</h4>
							<div class="fm-form-control">
								<input class="fm-textarea callback-url w-100" readonly name="callback_url" type="search" value="https://api.flowmattic.com/oauth2/callback">
							</div>
						</div>
						<div class="mb-3">
							<h4 class="fm-input-title"><?php esc_attr_e( 'Auth URL', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></h4>
							<div class="fm-form-control">
								<textarea class="form-control fm-textarea w-100" name="authUrl" rows="1" required type="search"></textarea>
							</div>
						</div>
						<div class="mb-3">
							<h4 class="fm-input-title"><?php esc_attr_e( 'Access Token URL', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></h4>
							<div class="fm-form-control">
								<textarea class="form-control fm-textarea w-100" name="tokenUrl" rows="1" required type="search"></textarea>
							</div>
						</div>
						<div class="mb-3">
							<h4 class="fm-input-title"><?php esc_attr_e( 'Client ID', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></h4>
							<div class="fm-form-control">
								<textarea class="form-control fm-textarea w-100" name="client_id" rows="1" required type="search"></textarea>
							</div>
						</div>
						<div class="mb-3">
							<h4 class="fm-input-title"><?php esc_attr_e( 'Client Secret', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></h4>
							<div class="fm-form-control">
								<textarea class="form-control fm-textarea w-100" name="client_secret" rows="1" required type="search"></textarea>
							</div>
						</div>
						<div class="mb-3">
							<h4 class="fm-input-title"><?php esc_attr_e( 'Scopes', 'flowmattic' ); ?></h4>
							<div class="fm-form-control">
								<textarea class="form-control fm-textarea w-100" name="scopes" rows="1" type="search"></textarea>
							</div>
						</div>
						<div class="mb-3">
							<h4 class="fm-input-title"><?php esc_attr_e( 'State', 'flowmattic' ); ?></h4>
							<div class="fm-form-control">
								<textarea class="form-control fm-textarea w-100" name="state" rows="1" type="search"></textarea>
							</div>
						</div>
						<div class="mb-3 auth_api_addto">
							<h4 class="fm-input-title"><?php esc_attr_e( 'Client Authentication', 'flowmattic' ); ?></h4>
							<select class="fm-select border bg-white w-100 mw-100" name="auth_api_addto">
								<option value="header"><?php esc_attr_e( 'Send as Basic Auth header', 'flowmattic' ); ?></option>
								<option value="body"><?php esc_attr_e( 'Send client credentials in body', 'flowmattic' ); ?></option>
							</select>
						</div>
					</div>
					<p><button type="submit" class="btn btn-save-auth btn-connect-auth btn-primary"><?php esc_html_e( 'Save Connect & Authenticate', 'flowmattic' ); ?></button></p>
				</script>
				<!-- Edit Connect Modal -->
				<div class="modal fade" id="external-connect-settings-modal" data-backdrop="static" data-keyboard="false" tabindex="-1" aria-labelledby="external-connect-settings-modal-label" aria-hidden="true">
					<div class="modal-dialog modal-lg modal-dialog-centered">
						<div class="modal-content">
							<form id="external-connect-edit-form" class="connect-external" novalidate>
								<div class="modal-header">
									<h5 class="modal-title" id="external-connect-settings-modal-label"><?php esc_html_e( 'Connect App:', 'flowmattic' ); ?> <span class="connect-name"></span></h5>
									<button type="button" class="btn-close" data-dismiss="modal" aria-label="<?php esc_html_e( 'Close', 'flowmattic' ); ?>"></button>
								</div>
								<div class="modal-body">
									<div class="form-group bg-light p-3 pb-1 mb-3 w-100">
										<div class="mb-3">
											<label for="external_connect_name" class="form-label"><?php esc_html_e( 'Connect Name', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></label>
											<input type="search" name="connect_name" class="form-control fm-textarea" id="external_connect_name" required>
											<input type="hidden" name="connect_id" class="form-control" id="connect_id" required>
											<input type="hidden" name="external_slug" class="form-control" id="external_slug" required>
											<div class="form-text"><?php esc_html_e( 'Name the connection, prefer API provider name for better understanding.', 'flowmattic' ); ?></div>
										</div>
									</div>
									<div class="external-connect-auth-form"></div>
								</div>
							</form>
						</div>
					</div>
				</div>
				<!-- External Connect - OAuth -->
				<script type="text/html" id="external-connect-oauth-form">
					<input type="hidden" name="endpoint_url" class="form-control" id="endpoint_url" required>
					<p><button type="submit" class="btn btn-save-auth btn-connect-auth btn-primary"><?php esc_html_e( 'Authenticate & Save Connect', 'flowmattic' ); ?></button></p>
				</script>
				<!-- External Connect - API Key -->
				<script type="text/html" id="external-connect-api-form">
					<div class="form-group bg-light p-3 pb-1 mb-3 api-url w-100">
						<div class="mb-3 auth_api_value">
							<h4 class="fm-input-title"><?php esc_attr_e( 'API Key Value', 'flowmattic' ); ?> <small class="badge text-danger bg-light"><?php esc_html_e( 'Required', 'flowmattic' ); ?></small></h4>
							<input class="form-control fm-textarea w-100" name="auth_api_value" type="search" autocomplete="new-password" placeholder="Eg: 338c5d58ffe9ea713c3e52ad1443e749" required/>
						</div>
					</div>
					<p><button type="submit" class="btn btn-save-auth btn-primary"><?php esc_html_e( 'Save Connect', 'flowmattic' ); ?></button></p>
				</script>
			</div>
		</div>
	</div>
</div>
<script type="text/javascript">
var externalConnects = <?php echo wp_json_encode( $external_connects ); ?>;
// Handle the form content.
document.addEventListener('DOMContentLoaded', function () {
	const selectElement = document.getElementById('fm-auth-type');
	const targetElement = document.querySelector('.auth-details-form');

	selectElement.addEventListener('change', function () {
		const selectedOptionValue = this.value;
		const selectedContent = document.getElementById( 'connect-type-' + selectedOptionValue + '-form');
		targetElement.innerHTML = '';
		if ( selectedContent ) {
			// Replace the form.
			targetElement.innerHTML = selectedContent.innerHTML;

			// Set the select picker.
			jQuery( '.fm-select' ).selectpicker();

			// Show tooltips
			jQuery( 'body' ).find( '[data-toggle="tooltip"]' ).tooltip( { template: '<div class="tooltip" role="tooltip"><div class="tooltip-inner"></div></div>' } );

			// Set the form as non-validated.
			jQuery( '#fm-connect-form' ).removeClass( 'was-validated' );
		}
	});
});

jQuery( document ).ready( function ($) {
	const swalPopup = window.Swal.mixin({
		customClass: {
			confirmButton: 'btn btn-primary shadow-none me-xxl-3',
			cancelButton: 'btn btn-danger shadow-none'
		},
		buttonsStyling: false
	} );
	
	// Handle the save and update auth actions.
	jQuery( '#fm-connect-form, #fm-connect-edit-form, #external-connect-edit-form' ).on( 'submit', function(e) {
		e.preventDefault();

		const form            = jQuery( this );
		const thisButton      = form.find( '.btn-save-auth' );
		const connectID       = form.find( '[name="connect_id"]' ).val();
		const connectFormData = new FormData( form[0] );
		const authConnect     = thisButton.hasClass( 'btn-connect-auth' );
		const isExternal      = form.hasClass( 'connect-external' );
		let authType          = form.find( 'select[name="fm_auth_type"]' ).val();

		form.addClass( 'was-validated' );

		if ( ! form[0].checkValidity() ) {
			return false;
		}

		// Add saving animation for button.
		thisButton.addClass( 'disabled' );
		thisButton.html( '<span class="spinner-grow spinner-grow-sm me-2" role="status" aria-hidden="true"></span>Saving...' );

		// Add action name and nonce to the form data.
		connectFormData.append( 'action', 'flowmattic_connect_save_auth_data' );
		connectFormData.append( 'workflow_nonce', FMConfig.workflow_nonce );
		connectFormData.append( 'oauth_webhook', FMConfig.oauth_webhook );

		if ( isExternal ) {
			let externalSlug = form.find( '[name="external_slug"]' ).val();
			let connectItem = window.externalConnects[ externalSlug ],
			authType = connectItem.fm_auth_type;
			connectFormData.append( 'is_external', true );
			connectFormData.append( 'external_slug', externalSlug );
		}

		if ( 'undefined' !== typeof connectID && '' !== connectID ) {
			connectFormData.append( 'connect_id', connectID );
		}

		if ( 'undefined' !== typeof authType ) {
			connectFormData.append( 'fm_auth_type', authType );
		}

		// Show saving popup.
		swalPopup.fire(
			{
				title: 'Saving Connect',
				showConfirmButton: false,
				didOpen: function() {
					swalPopup.showLoading();
				}
			}
		);

		jQuery.ajax({
			url: ajaxurl,
			type: 'POST',
			processData: false,
			contentType: false,
			data: connectFormData,
			success: function( response ) {
				var authWindow;

				response = JSON.parse( response );
				const connectID = response.connect_id;

				// Show finished popup.
				swalPopup.fire(
					{
						title: 'Connect Saved!',
						text: 'Your new connect is saved successfully.',
						icon: 'success',
						showConfirmButton: false,
						timer: 1500
					}
				);

				if ( authConnect ) {
					var authFormData = form.serializeArray(),
						authDataCapture = setInterval( captureConnectAuthData, 2000 ),
						captureResponse = 1;

					const baseURL = isExternal ? form.find( '#endpoint_url' ).val() : form.find( '.callback-url' ).val();
					
					// Convert form data array to JSON
					var formDataJSON = authFormData.reduce((result, { name, value }) => {
						result[name] = value;
						return result;
					}, {});

					// Add the required params.
					formDataJSON.connect_id    = connectID;
					formDataJSON.oauth_webhook = FMConfig.oauth_webhook;

					// Convert Object to String.
					formDataJSON = btoa( JSON.stringify( formDataJSON ) );

					// Append it to the callback URL.
					const callbackURL = `${baseURL}?connect=${formDataJSON}`;

					// Set popup window dimensions.
					const left = (window.innerWidth / 2) - 400;
					const top = (window.innerHeight / 2) - 400;

					// Show authenticating text.
					thisButton.html( '<span class="spinner-grow spinner-grow-sm me-2" role="status" aria-hidden="true"></span>Authenticating...' );

					// Show authenticating popup.
					swalPopup.fire(
						{
							title: 'Processing Authentication',
							text: "This dialog box will automatically close once the auth callback is received. Make sure your browser isn't blocking pop-up windows.",
							showConfirmButton: false,
							didOpen: function() {
								swalPopup.showLoading();
							}
						}
					);

					authWindow = window.open(
						callbackURL,
						'_blank',
						`width=800,height=800,top=${top},left=${left}`
					);
					authWindow.focus();

					function captureConnectAuthData() {
						jQuery.ajax( {
							url: ajaxurl,
							type: 'POST',
							data: { action: 'flowmattic_connect_capture_data', 'connect_id': connectID, workflow_nonce: FMConfig.workflow_nonce, capture: captureResponse },
							success: function( response ) {
								response = JSON.parse( response );

								if ( 'pending' !== response.status ) {
									clearInterval( authDataCapture );

									// Show reloading text.
									thisButton.html( '<span class="spinner-grow spinner-grow-sm me-2" role="status" aria-hidden="true"></span>Refreshing page...' );

									// Show success popup.
									swalPopup.fire(
										{
											title: 'Authentication Successful!',
											text: 'Your new connect is authenticated successfully. You can now use it in your workflow.',
											icon: 'success',
											showConfirmButton: false,
											timer: 1500
										}
									);

									setTimeout( function() {
										window.location = window.location.href;
									}, 500 );
								} else {
									if ( authWindow.closed ) {
										clearInterval( authDataCapture );

										swalPopup.fire(
											{
												title: 'Authentication Cancelled!',
												text: 'You have cancelled the authentication. Please authenticate your connect to be able to use this in your workflow automation.',
												icon: 'warning',
												showConfirmButton: true,
												timer: 5000
											}
										);

										// Show reloading text.
										thisButton.html( '<span class="spinner-grow spinner-grow-sm me-2" role="status" aria-hidden="true"></span>Refreshing page...' );

										setTimeout( function() {
											window.location = window.location.href;
										}, 500 );

										return false;
									}
								}

								captureResponse = '0';
							}
						} );
					}
				} else {
					// Show reloading text.
					thisButton.html( '<span class="spinner-grow spinner-grow-sm me-2" role="status" aria-hidden="true"></span>Refreshing page...' );

					setTimeout( function() {
						window.location = window.location.href;
					}, 500 );
				}
			},
			error: function (xhr, status, error) {
				console.log('Error:', error);
			}
		});
	});

	// Edit connect settings.
	jQuery( '.btn-edit-connect' ).on( 'click', function() {
		var connectID = jQuery( this ).attr( 'data-connect-id' );

		// Show preparing popup.
		swalPopup.fire(
			{
				title: 'Preparing to edit',
				text: 'Please wait while we fetch the connect data.',
				showConfirmButton: false,
				didOpen: function() {
					swalPopup.showLoading();
				}
			}
		);

		// Fetch the connect settings.
		jQuery.ajax( {
			url: ajaxurl,
			type: 'POST',
			data: { action: 'flowmattic_connect_edit_settings', workflow_nonce: FMConfig.workflow_nonce, connect_id: connectID },
			success: function( response ) {
				response = JSON.parse( response );

				if ( 'undefined' !== typeof response.status ) {
					// Show error popup.
					swalPopup.fire(
						{
							title: 'Error',
							text: response.message,
							icon: 'warning',
							showConfirmButton: true,
						}
					);
				} else {
					var auth_type = response.connect_settings.fm_auth_type,
						targetElement = jQuery( '#edit-connect-settings-modal' ).find( '.auth-details-form' ),
						selectedContent = document.getElementById( 'connect-type-' + auth_type + '-form');

					if ( 'undefined' !== typeof response.connect_settings.is_external ) {
						var connectSlug   = response.connect_settings.external_slug,
							modal = jQuery( '#external-connect-settings-modal' ),
							connectItem = window.externalConnects[ connectSlug ],
							authType = connectItem.fm_auth_type,
							authButton;

						if ( 'oauth' === authType || 'api' === authType ) {
							selectedContent = document.getElementById( 'external-connect-' + authType + '-form');
						}

						if ( 'bearer' === authType ) {
							selectedContent = document.getElementById( 'connect-type-bearer-form');
						}

						if ( 'basic' === authType ) {
							selectedContent = document.getElementById( 'connect-type-basic-form');
						}

						targetElement = modal.find( '.external-connect-auth-form' );
						targetElement.innerHTML = '';

						// Replace the form.
						targetElement[0].innerHTML = selectedContent.innerHTML;

						// Set the connect name.
						modal.find( '[name="connect_name"]' ).val( response.connect_name );
						modal.find( '.connect-name' ).html( response.connect_name );

						if ( 'oauth' === authType ) { 
							// Set the connect endpoint URL.
							modal.find( '[name="endpoint_url"]' ).val( response.connect_settings.endpoint_url );
						} else if ( 'api' === authType ) {
							// Set the external slug.
							modal.find( '[name="auth_api_value"]' ).val( response.connect_settings.auth_api_value );
						} else if ( 'basic' === authType ) {
							// Set the api key.
							modal.find( '[name="auth_api_key"]' ).val( response.connect_settings.auth_api_key );

							// Set the api secret.
							modal.find( '[name="auth_api_secret"]' ).val( response.connect_settings.auth_api_secret );
						} else if ( 'bearer' === authType ) {
							// Set the bearer token.
							modal.find( '[name="auth_bearer_token"]' ).val( response.connect_settings.auth_bearer_token );
						}

						// Set the external slug.
						modal.find( '[name="external_slug"]' ).val( response.connect_settings.external_slug );

						// Set the connect ID.
						modal.find( '[name="connect_id"]' ).val( connectID );

						// Show the modal.
						modal.modal( 'show' );
					} else {
						// Replace the form.
						jQuery( targetElement ).html( selectedContent.innerHTML );

						// Set all fields from the settings.
						jQuery.each( response.connect_settings, function( setting, value ) {
							jQuery( '#edit-connect-settings-modal' ).find( '[name="' + setting + '"]' ).val( value );
						} );

						// Set the connect ID.
						jQuery( '#edit-connect-settings-modal' ).find( '[name="connect_id"]' ).val( connectID );

						// Set the connect name.
						jQuery( '#edit-connect-settings-modal' ).find( '[name="connect_name"]' ).val( response.connect_name );
						jQuery( '#edit-connect-settings-modal' ).find( '.connect-name' ).html( response.connect_name );

						// Set the auth type.
						jQuery( '#edit-connect-settings-modal' ).find( 'select[name="fm_auth_type"]' ).selectpicker( 'val', response.connect_settings.fm_auth_type );

						// Set the select picker.
						jQuery( '.fm-select' ).selectpicker();

						// Show tooltips
						jQuery( 'body' ).find( '[data-toggle="tooltip"]' ).tooltip( { template: '<div class="tooltip" role="tooltip"><div class="tooltip-inner"></div></div>' } );

						// Set the form as non-validated.
						jQuery( '#fm-connect-form' ).removeClass( 'was-validated' );

						
						// Show the modal.
						jQuery( '#edit-connect-settings-modal' ).modal( 'show' );
					}

					// Close the popup.
					swalPopup.close();
				}
			}
		} );
	} );

	// Rename Connect form.
	jQuery( '.btn-rename-connect' ).on( 'click', function( e ) {
		var connectID   = jQuery( this ).attr( 'data-connect-id' ),
			connectName = jQuery( this ).attr( 'data-connect-name' );

		// Set the connect name.
		jQuery( '#fm-connect-rename-modal' ).find( '[name="connect_name"]' ).val( connectName );

		// Set the connect ID.
		jQuery( '#fm-connect-rename-modal' ).find( '[name="connect_id"]' ).val( connectID );

		// Show the modal.
		jQuery( '#fm-connect-rename-modal' ).modal( 'show' );
	} );

	// Rename connect.
	jQuery( '#fm-connect-rename' ).on( 'submit', function(e) {
		e.preventDefault();

		const form            = jQuery( this );
		const thisButton      = form.find( '.btn-rename-auth' );
		const connectID       = form.find( '[name="connect_id"]' ).val();
		const connectName     = form.find( '[name="connect_name"]' ).val();
		const connectFormData = new FormData( form[0] );
		const connectRow      = jQuery( '.connect-item-' + connectID );

		// Show saving popup.
		swalPopup.fire(
			{
				title: 'Saving Connect',
				showConfirmButton: false,
				didOpen: function() {
					swalPopup.showLoading();
				}
			}
		);

		// Add action name and nonce to the form data.
		connectFormData.append( 'action', 'flowmattic_connect_save_auth_data' );
		connectFormData.append( 'workflow_nonce', FMConfig.workflow_nonce );
		connectFormData.append( 'connect_rename', true );

		// Add saving animation for button.
		thisButton.addClass( 'disabled' );
		thisButton.html( '<span class="spinner-grow spinner-grow-sm me-2" role="status" aria-hidden="true"></span>Renaming...' );

		jQuery.ajax({
			url: ajaxurl,
			type: 'POST',
			processData: false,
			contentType: false,
			data: connectFormData,
			success: function( response ) {
				// Show finished popup.
				swalPopup.fire(
					{
						title: 'Connect Renamed!',
						text: 'New name for the connect is saved successfully.',
						icon: 'success',
						showConfirmButton: false,
						timer: 1500
					}
				);

				// Update the name on the page.
				connectRow.find( '.connect-name' ).html( connectName );
				connectRow.find( '.btn-rename-connect' ).attr( 'data-connect-name', connectName );

				// Hide the modal.
				jQuery( '#fm-connect-rename-modal' ).modal( 'hide' );

				// Remove animation for button and enable it.
				thisButton.html( 'Rename' );
				thisButton.removeClass( 'disabled' );
			}
		} );
	} );

	// Delete Connect.
	jQuery( '.btn-delete-connect' ).on( 'click', function( e ) {
		var connectID = jQuery( this ).attr( 'data-connect-id' );

		swalPopup.fire( {
			title: 'Are you sure?',
			text: "Once the selected connect is deleted, your workflows using this connect will be paused until you manually inspect and publish them with new connect for authentication.",
			icon: 'warning',
			showCancelButton: true,
			confirmButtonText: 'Yes, delete it!',
			showLoaderOnConfirm: true,
		} ).then( function( result ) {
				if ( result.isConfirmed ) {
				// Show loading.
				swalPopup.fire(
					{
						title: 'Deleting Selected Connect',
						text: 'Please wait while we delete the selected connect. Page will be refreshed once its done.',
						showConfirmButton: false,
						didOpen: function() {
							swalPopup.showLoading();
						}
					}
				);

				// Process delete connect ajax.
				jQuery.ajax( {
					url: ajaxurl,
					type: 'POST',
					data: { action: 'flowmattic_connect_delete', workflow_nonce: FMConfig.workflow_nonce, connect_id: connectID },
					success: function( response ) {
						// Show success popup.
						swalPopup.fire(
							{
								title: 'Connect Deleted!',
								text: 'Selected connect is deleted successfully.',
								icon: 'success',
								showConfirmButton: false,
								timer: 1500
							}
						);

						setTimeout( function() {
							window.location = window.location.href;
						}, 500 );
					}
				} );
			}
		} );
	} );

	// Process external connect authentication.
	jQuery( '.external-connect-auth-button' ).on( 'click', function( e ) {
		var connectSlug   = jQuery( this ).attr( 'data-connect-slug' ),
			modal = jQuery( '#external-connect-settings-modal' ),
			connectItem = window.externalConnects[ connectSlug ],
			authType = connectItem.fm_auth_type,
			authButton;

		let selectedContent = document.getElementById( 'external-connect-' + authType + '-form');
		let targetElement = modal.find( '.external-connect-auth-form' )
		targetElement.innerHTML = '';

		if ( 'bearer' === authType ) {
			selectedContent = document.getElementById( 'connect-type-bearer-form');
		}

		if ( 'basic' === authType ) {
			selectedContent = document.getElementById( 'connect-type-basic-form');
		}

		if ( 'api' === authType ) {
			selectedContent = document.getElementById( 'connect-type-api-form-external');
		}

		// Replace the form.
		targetElement[0].innerHTML = selectedContent.innerHTML;

		// Remove the connect id.
		modal.find( '[name="connect_id"]' ).val('');

		// Set the connect name.
		modal.find( '[name="connect_name"]' ).val( connectItem.name );
		modal.find( '.connect-name' ).html( connectItem.name );

		// Set the external slug.
		modal.find( '[name="external_slug"]' ).val( connectSlug );

		if ( 'oauth' === authType ) {
			// Set the connect endpoint URL.
			modal.find( '[name="endpoint_url"]' ).val( connectItem.endpoint );
		}

		// Show the modal.
		modal.modal( 'show' );

		jQuery( '#choose-connect-external-modal' ).modal( 'hide' );
	} );
});
</script>
<?php FlowMattic_Admin::footer(); ?>
